MODULE p4zflx
   !!======================================================================
   !!                         ***  MODULE p4zflx  ***
   !! TOP :   PISCES CALCULATES GAS EXCHANGE AND CHEMISTRY AT SEA SURFACE
   !!======================================================================
   !! History :    -   !  1988-07  (E. MAIER-REIMER) Original code
   !!              -   !  1998     (O. Aumont) additions
   !!              -   !  1999     (C. Le Quere) modifications
   !!             1.0  !  2004     (O. Aumont) modifications
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces
   !!----------------------------------------------------------------------
   !!   'key_pisces'                                       PISCES bio-model
   !!----------------------------------------------------------------------
   !!   p4z_flx       :   CALCULATES GAS EXCHANGE AND CHEMISTRY AT SEA SURFACE
   !!   p4z_flx_init  :   Read the namelist
   !!----------------------------------------------------------------------
   USE trc
   USE oce_trc         !
   USE trp_trc
   USE sms
   USE prtctl_trc
   USE p4zche

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_flx    ! called in p4zprg.F90

   REAL(wp) :: &  ! pre-industrial atmospheric [co2] (ppm) 	
     atcox  = 0.20946 ,   &  !:
     atcco2 = 278.           !:

   REAL(wp) :: &
     tco2flx = 0.            !: Total flux of carbon per year

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_flx ( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_flx  ***
      !!
      !! ** Purpose :   CALCULATES GAS EXCHANGE AND CHEMISTRY AT SEA SURFACE
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER, INTENT(in) :: kt
      INTEGER  ::   ji, jj, jrorr, nspyr
      REAL(wp) ::   zttc, zws, zkgwan
      REAL(wp) ::   zfld, zflu, zfld16, zflu16, zfact
      REAL(wp) ::   zph, zah2, zbot, zdic, zalk, zschmitto2, zalka, zschmittco2
      REAL(wp), DIMENSION(jpi,jpj) ::   zkgco2, zkgo2, zh2co3, ztco2flx
      CHARACTER (len=25) :: charout

      !!---------------------------------------------------------------------


      IF( kt == nittrc000  )   CALL p4z_flx_init      ! Initialization (first time-step only)


      ! -----------------------------------------------------
      !     ASSIGNATION TO EXPONENTS IN THE LISS AND MERLIVAT
      !     FORMULATION OF THE GAS EXCHANGE RATE
      ! -----------------------------------------------------

      nspyr = INT( raass / rdt )

      ! SURFACE CHEMISTRY (PCO2 AND [H+] IN
      !     SURFACE LAYER); THE RESULT OF THIS CALCULATION
      !     IS USED TO COMPUTE AIR-SEA FLUX OF CO2

      DO jrorr = 1, 10

!CDIR NOVERRCHK
         DO jj = 1, jpj
!CDIR NOVERRCHK
            DO ji = 1, jpi

               ! DUMMY VARIABLES FOR DIC, H+, AND BORATE
               zbot  = borat(ji,jj,1)
               zfact = rhop(ji,jj,1) / 1000. + rtrn
               zdic  = trn(ji,jj,1,jpdic) / zfact
               zph   = MAX( hi(ji,jj,1), 1.e-10 ) / zfact
               zalka = trn(ji,jj,1,jptal) / zfact

               ! CALCULATE [ALK]([CO3--], [HCO3-])
               zalk  = zalka - (  akw3(ji,jj,1) / zph - zph + zbot / ( 1.+ zph / akb3(ji,jj,1) )  )

               ! CALCULATE [H+] AND [H2CO3]
               zah2   = SQRT(  (zdic-zalk)**2 + 4.* ( zalk * ak23(ji,jj,1)   &
                  &                                        / ak13(ji,jj,1) ) * ( 2.* zdic - zalk )  )
               zah2   = 0.5 * ak13(ji,jj,1) / zalk * ( ( zdic - zalk ) + zah2 )
               zh2co3(ji,jj) = ( 2.* zdic - zalk ) / ( 2.+ ak13(ji,jj,1) / zah2 ) * zfact
               hi(ji,jj,1)   = zah2 * zfact
            END DO
         END DO
      END DO


      ! --------------
      ! COMPUTE FLUXES
      ! --------------

      ! FIRST COMPUTE GAS EXCHANGE COEFFICIENTS
      ! -------------------------------------------

!CDIR NOVERRCHK
      DO jj = 1, jpj
!CDIR NOVERRCHK
         DO ji = 1, jpi

            zttc = MIN( 35., tn(ji,jj,1) )

      ! Compute the schmidt Number both O2 and CO2
      ! ------------------------------------------

            zschmittco2 = 2073.1 - 125.62 * zttc + 3.6276 * zttc**2 - 0.043126 * zttc**3
            zschmitto2  = 1953.4 - 128.0 * zttc + 3.9918 * zttc**2 - 0.050091 * zttc**3

            !  wind speed 
            zws  = wndm(ji,jj)

      ! Compute the piston velocity for O2 and CO2
      ! ------------------------------------------

            zkgwan = (  0.3 * zws * zws    &
               &   + 2.5 * ( 0.5246 + zttc * ( 0.016256+zttc*0.00049946 ) ) ) &
# if defined key_off_degrad
               &         * facvol(ji,jj,1)      &
# endif
               &   / (100. * 3600.)* ( 1.- fr_i(ji,jj) ) * tmask(ji,jj,1)  

            ! COMPUTE GAS EXCHANGE FOR CO2
            zkgco2(ji,jj) = zkgwan * SQRT( 660./ zschmittco2 )
            zkgo2(ji,jj)  = zkgwan * SQRT( 660./ zschmitto2 )

         END DO
      END DO

      ztco2flx(:,:) = 0.
      DO jj = 1, jpj
         DO ji = 1, jpi

            ! Compute CO2 flux for the sea and air
            zfld = atcco2 * tmask(ji,jj,1) * chemc(ji,jj,1) * zkgco2(ji,jj)
            zflu = zh2co3(ji,jj) * tmask(ji,jj,1) * zkgco2(ji,jj)
            tra(ji,jj,1,jpdic) = tra(ji,jj,1,jpdic) + ( zfld - zflu ) / fse3t(ji,jj,1)

            ! compute flux of carbon
            ztco2flx(ji,jj) = ( zfld - zflu ) * rfact &
               &             * e1t(ji,jj) * e2t(ji,jj) * tmask(ji,jj,1) * 1000.

            ! Compute O2 flux 
            zfld16 = atcox * chemc(ji,jj,2) *tmask(ji,jj,1) * zkgo2(ji,jj)
            zflu16 = trn(ji,jj,1,jpoxy) * tmask(ji,jj,1) * zkgo2(ji,jj)
            tra(ji,jj,1,jpoxy) = tra(ji,jj,1,jpoxy) + ( zfld16 - zflu16 ) / fse3t(ji,jj,1)

# if defined key_trc_diaadd
            ! Save diagnostics
            trc2d(ji,jj,1) = ( zfld - zflu ) * 1000.
            trc2d(ji,jj,2) = ( zfld16 - zflu16 ) * 1000.
            trc2d(ji,jj,3) = zkgco2(ji,jj)
            trc2d(ji,jj,4) = atcco2 - zh2co3(ji,jj) / ( chemc(ji,jj,1) + rtrn )
# endif
         END DO
      END DO
      !

      ! Total Flux of Carbon
      DO jj = 1, jpj
        DO ji = 1, jpi
           tco2flx = tco2flx + ztco2flx(ji,jj) * tmask_i(ji,jj)
        END DO
      END DO

      IF( MOD( kt, nspyr ) == 0 ) THEN
        WRITE(numout,*) ' Atmospheric pCO2    :'
        WRITE(numout,*) '-------------------- : ',kt,'  ',atcco2
        WRITE(numout,*) '(ppm)'
        WRITE(numout,*) 'Total Flux of Carbon :'
        WRITE(numout,*) '-------------------- : ',tco2flx * 12./1E15
        WRITE(numout,*) '(GtC/an)'
        tco2flx = 0.
      ENDIF

      IF(ln_ctl)   THEN  ! print mean trends (used for debugging)
         WRITE(charout, FMT="('flx ')")
         CALL prt_ctl_trc_info(charout)
         CALL prt_ctl_trc(tab4d=tra, mask=tmask, clinfo=ctrcnm)
      ENDIF


   END SUBROUTINE p4z_flx

   SUBROUTINE p4z_flx_init

      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE p4z_flx_init  ***
      !!
      !! ** Purpose :   Initialization of atmospheric conditions
      !!
      !! ** Method  :   Read the natext namelist and check the parameters
      !!      called at the first timestep (nittrc000)
      !! ** input   :   Namelist natext
      !!
      !!----------------------------------------------------------------------

      NAMELIST/natext/ atcco2

      REWIND( numnat )                     ! read numnat
      READ  ( numnat, natext )

      IF(lwp) THEN                         ! control print
         WRITE(numout,*) ' '
         WRITE(numout,*) ' Namelist parameters for air-sea exchange, natext'
         WRITE(numout,*) ' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
         WRITE(numout,*) '    Atmospheric pCO2      atcco2      =', atcco2
      ENDIF

   END SUBROUTINE p4z_flx_init

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_flx( kt )                   ! Empty routine
      INTEGER, INTENT( in ) ::   kt
      WRITE(*,*) 'p4z_flx: You should not have seen this print! error?', kt
   END SUBROUTINE p4z_flx
#endif 

   !!======================================================================
END MODULE  p4zflx
